<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 20/04/20
 * Time: 11.57
 */

namespace FiloBlu\Rma\Block\Returns;
use Magento\Framework\App\ProductMetadataInterface;
use FiloBlu\Rma\Helper\EasyReturnHelper;

class Create extends \Magento\Rma\Block\Returns\Create
{
    /**
     * @var ProductMetadataInterface
     */
    private $productMetadata;

    const CREATE_TEMPLATE_MAGENTO_V2_3_X = 'FiloBlu_Rma::return/createv23x.phtml';

    const CREATE_TEMPATE_MAGENTO_ORIGINAL = 'Magento_Rma::return/create.phtml';
    /**
     * @var EasyReturnHelper
     */
    private $easyReturnHelper;
    /**
     * @var \Magento\Framework\View\Element\Template\Context
     */
    private $context;

    /**
     * Create constructor.
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \Magento\Framework\Data\Collection\ModelFactory $modelFactory
     * @param \Magento\Eav\Model\Form\Factory $formFactory
     * @param \Magento\Eav\Model\Config $eavConfig
     * @param \Magento\Rma\Model\ItemFactory $itemFactory
     * @param \Magento\Rma\Model\Item\FormFactory $itemFormFactory
     * @param \Magento\Rma\Helper\Data $rmaData
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Sales\Model\Order\Address\Renderer $addressRenderer
     * @param ProductMetadataInterface $productMetadata
     * @param EasyReturnHelper $easyReturnHelper
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Framework\Data\Collection\ModelFactory $modelFactory,
        \Magento\Eav\Model\Form\Factory $formFactory,
        \Magento\Eav\Model\Config $eavConfig,
        \Magento\Rma\Model\ItemFactory $itemFactory,
        \Magento\Rma\Model\Item\FormFactory $itemFormFactory,
        \Magento\Rma\Helper\Data $rmaData,
        \Magento\Framework\Registry $registry,
        \Magento\Sales\Model\Order\Address\Renderer $addressRenderer,
        ProductMetadataInterface $productMetadata,
        EasyReturnHelper $easyReturnHelper,
        array $data = [])
    {
        parent::__construct($context, $modelFactory, $formFactory, $eavConfig, $itemFactory, $itemFormFactory, $rmaData, $registry, $addressRenderer, $data);
        $this->productMetadata = $productMetadata;
        $this->easyReturnHelper = $easyReturnHelper;

        $this->context = $context;
    }

    /**
     * @param string $template
     * @return \Magento\Rma\Block\Returns\Create
     */
    public function setTemplate($template){
        try{
            if(!$this->easyReturnHelper->isEasyReturnEnabled($this->context->getStoreManager()->getStore()->getId())){
                return parent::setTemplate(self::CREATE_TEMPATE_MAGENTO_ORIGINAL);
            }

            $magentoVersion = $this->productMetadata->getVersion();
            if(version_compare($magentoVersion,  "2.3.0") != -1){
                $template = self::CREATE_TEMPLATE_MAGENTO_V2_3_X;
            }
            return parent::setTemplate($template);
        }catch (\Exception $e){
            $this->_logger->critical($e->getMessage());
            return parent::setTemplate(self::CREATE_TEMPATE_MAGENTO_ORIGINAL);
        }catch (\Throwable $t){
            $this->_logger->critical($t->getMessage());
            return parent::setTemplate(self::CREATE_TEMPATE_MAGENTO_ORIGINAL);
        }
    }

}