<?php


namespace FiloBlu\Rma\Model;
use FiloBlu\Rma\Api\Data\EasyReturnInterface;
use Magento\Framework\DataObject\IdentityInterface;
use Magento\Framework\Model\AbstractModel;
use Magento\Sales\Api\Data\OrderAddressInterfaceFactory;
use FiloBlu\Rma\Helper\EasyReturnHelper;

class EasyReturn extends AbstractModel implements
    IdentityInterface,
    EasyReturnInterface
{
    const CACHE_TAG = 'filoblu_rma_easyreturn';

    protected $_cacheTag = 'filoblu_rma_easyreturn';

    protected $_eventPrefix = 'filoblu_rma_easyreturn';

    protected $entityId;
    protected $customerFirstname;
    protected $customerLastname;
    protected $rmaEntityId;
    protected $street;
    protected $city;
    protected $regionId;
    protected $region;
    protected $postcode;
    protected $countryId;
    protected $theoricShippingAmount;
    /**
     * @var Context
     */
    private $context;
    /**
     * @var OrderAddressInterfaceFactory
     */
    private $addressInterfaceFactory;
    /**
     * @var EasyReturnHelper
     */
    private $easyReturnHelper;


    protected function _construct()
    {
        $this->_init('FiloBlu\Rma\Model\ResourceModel\EasyReturn');
    }


    /**
     * EasyReturn constructor.
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param OrderAddressInterfaceFactory $addressInterfaceFactory
     * @param EasyReturnHelper $easyReturnHelper
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        OrderAddressInterfaceFactory $addressInterfaceFactory,
        EasyReturnHelper $easyReturnHelper,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = [])
    {
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
        $this->context = $context;
        $this->addressInterfaceFactory = $addressInterfaceFactory;
        $this->easyReturnHelper = $easyReturnHelper;
    }

    /**
     * @return array|string[]
     */
    public function getIdentities()
    {
        return [self::CACHE_TAG . '_' . $this->getId()];
    }


    /**
     * @param string $firstname
     * @return $this
     */
    public function setFirstName(string $firstname)
    {
        return $this->setData(self::FIRSTNAME, $firstname);
    }

    /**
     * @return string
     */
    public function getFirstName()
    {
        return $this->getData(self::FIRSTNAME);
    }

    /**
     * @param string $lastname
     * @return $this
     */
    public function setLastName(string $lastname)
    {
       return $this->setData(self::LASTNAME,$lastname);
    }

    /**
     * @return string
     */
    public function getLastName()
    {
        return $this->getData(self::LASTNAME);
    }

    /**
     * @param int $rmaEntityId
     * @return $this
     */
    public function setRmaEntityId(int $rmaEntityId)
    {
        return $this->setData(self::RMA_ENTITY_ID,$rmaEntityId);
    }

    /**
     * @return int
     */
    public function getRmaEntityId()
    {
        return $this->getData(self::RMA_ENTITY_ID);
    }

    /**
     * @param string $street
     * @return $this
     */
    public function setStreet(string $street)
    {
        return $this->setData(self::STREET,$street);
    }

    /**
     * @return string
     */
    public function getStreet()
    {
        return $this->getData(self::STREET);
    }

    /**
     * @param string $city
     * @return $this
     */
    public function setCity(string $city)
    {
        return $this->setData(self::CITY,$city);
    }

    /**
     * @return string
     */
    public function getCity()
    {
        return $this->getData(self::CITY);
    }

    /**
     * @param int $regionId
     * @return $this
     */
    public function setRegionId(int $regionId)
    {
        return $this->setData(self::REGION_ID,$regionId);
    }

    /**
     * @return int
     */
    public function getRegionId()
    {
        return $this->getData(self::REGION_ID);
    }

    /**
     * @param string $region
     * @return $this
     */
    public function setRegion(string $region)
    {
        return $this->setData(self::REGION,$region);
    }

    /**
     * @return string
     */
    public function getRegion()
    {
        return $this->getData(self::REGION);
    }

    /**
     * @param string $postcode
     * @return $this
     */
    public function setPostcode(string $postcode)
    {
        return $this->setData(self::POSTCODE,$postcode);
    }

    /**
     * @return string
     */
    public function getPostcode()
    {
        return $this->getData(self::POSTCODE);
    }

    /**
     * @param string $countryId
     * @return $this
     */
    public function setCountryId(string $countryId)
    {
        return $this->setData(self::COUNTRY_ID,$countryId);
    }

    /**
     * @return string
     */
    public function getCountryId()
    {
        return $this->getData(self::COUNTRY_ID);
    }

    /**
     * @param float $amount
     * @return $this
     */
    public function setTheoricShippingAmount(float $amount)
    {
        return $this->setData(self::THEORIC_SHIPPING_AMOUNT,$amount);
    }

    /**
     * @return float
     */
    public function getTheoricShippingAmount()
    {
        return $this->getData(self::THEORIC_SHIPPING_AMOUNT);
    }

    /**
     * @return \Magento\Sales\Api\Data\OrderAddressInterface
     */
    public function getPickUpAddress()
    {
        $address = $this->addressInterfaceFactory->create();
        $address->setCountryId($this->getCountryId());
        $address->setCity($this->getCity())
            ->setRegionId($this->getRegionId())
            ->setPostcode($this->getPostcode())
            ->setFirstname($this->getFirstName())
            ->setLastname($this->getLastName())
            ->setRegion($this->getRegion())
            ->setStreet($this->getStreet());

        return $address;
    }

    /**
     * @param array $data
     * @return $this|EasyReturnInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function setEasyReturnData(array $data)
    {
         $this->setData($data);
         //save theoric shipping amount
        $theoricShippingAmount = $this->easyReturnHelper->simulateEasyReturnShippingCost($this->getPickUpAddress(),$data['store'],$data['weight']);
        $this->setTheoricShippingAmount($theoricShippingAmount);
        return $this;
    }


}